@@ -48,8 +48,8 @@ extern int regex_comp(exec_t *ex, void *p, char *s, int n, int f);
 extern int regex_exec(exec_t *ex, void *p, char *s, int n, int i, int so[10], int eo[10]);
 extern void regex_trace(exec_t* ex, void *p);
 extern void regex_free(exec_t* ex, void *p);
-extern void* regex_alloc(exec_t* ex);
 extern void regex_clear(exec_t* ex, void* p);
+extern int new_regex (exec_t* ex, var_t* v, char *s, int n, int i);
 extern void* array_alloc(exec_t* ex);
 extern void array_free(exec_t* ex, void *v);
 extern int array_push(exec_t* ex, array_t* arr, var_t* v);
@@ -78,6 +78,7 @@ extern void refer_clear(exec_t* ex, void *p);
 extern void refer_trace(exec_t* ex, void *p);
 extern int refer_reserv(exec_t* ex, array_t *arr, var_t* obj, var_t* key);
 extern int refer_set(exec_t* ex, var_t* v, var_t* val);
+extern int is_refer_var(var_t* v);
 extern int refer_resolv(exec_t* ex, array_t *arr, var_t* obj, var_t* key);
 extern int refer_get(exec_t* ex, var_t* v, var_t* res);
 extern int refer_del(exec_t* ex, var_t* v, var_t* res);
@@ -115,15 +116,20 @@ extern int frame_set(exec_t* ex, object_t* fp, int k, var_t* v);
 extern int frame_getItem(exec_t* ex, void *p, var_t* k, var_t* v);
 extern int frame_delItem(exec_t* ex, void *p, var_t* k, var_t* v);
 extern int frame_setItem(exec_t* ex, void *p, var_t* k, var_t* v);
+extern int new_file(exec_t* ex, var_t* v, FILE* fp, int (*xp)(FILE*));
+extern void file_clear(exec_t* ex, void* p);
+extern void file_trace(exec_t* ex, void *p);
 extern int file_callFunc(exec_t* ex, void* p, int sym, int argc, var_t* argv, var_t* res);
-extern int dbm_open(exec_t* ex, var_t* v, char* path, char* mode);
+extern int new_dbm(exec_t* ex, var_t* v, char* path, char* mode);
+extern void dbm_clear(exec_t* ex, void *p);
+extern void dbm_trace(exec_t* ex, void *p);
 extern int dbm_getItem(exec_t* ex, void* p, var_t* k, var_t* v);
 extern int dbm_setItem(exec_t* ex, void* p, var_t* k, var_t* v);
 extern int dbm_delItem(exec_t* ex, void* p, var_t* k, var_t* v);
 extern int dbm_eachItem(exec_t* ex, void* p, int i, var_t* k, var_t* v);
 extern int dbm_callFunc(exec_t* ex, void* p, int sym, int argc, var_t* argv, var_t* res);
-extern int xml_create(exec_t* ex, var_t* v, char *s, int n, char *f);
-extern int xml_open(exec_t* ex, var_t* v, char *filename, char* mode);
+extern int new_xml_from_string(exec_t* ex, var_t* v, char *s, int n, char *f);
+extern int new_xml_from_file(exec_t* ex, var_t* v, char *filename, char* mode);
 extern void xml_clear(exec_t* ex, void *p);
 extern void xml_trace(exec_t* ex, void *p);
 extern int xml_toString(exec_t* ex, var_t* v, char **s, int *n);
@@ -163,24 +169,22 @@ extern int var_setItem(exec_t* ex, var_t* v, var_t* key, var_t* val);
 extern int var_delItem(exec_t* ex, var_t* v, var_t* key, var_t* res);
 extern int var_eachItem(exec_t* ex, var_t* v, int i, var_t* key, var_t* val);
 extern int var_callFunc(exec_t* ex, var_t* v, int sym, int argc, var_t* argv, var_t* res);
-extern void new_null (var_t* v);
-extern void new_bool(var_t* v, int i);
-extern void new_char (var_t* v, int i);
-extern void new_int (var_t* v, int i);
-extern void new_symbol (var_t* v, int i);
-extern void new_float (var_t* v, double f);
-extern void new_label (var_t* v, char *p);
-extern void new_function (var_t* v, char *p);
-extern void new_class (var_t* v, char *p);
-extern void new_file(var_t* v, FILE* fp);
-extern void new_string_with (exec_t* ex, var_t* v, char *s, int n);
-extern void new_string_copy (exec_t* ex, var_t* v, char *s, int n);
-extern void new_string (exec_t* ex, var_t* v, void *p);
-extern void new_regex (exec_t* ex, var_t* v, char *s, int n, int i);
-extern void new_array (exec_t* ex, var_t* v, void *p);
-extern void new_refer (exec_t* ex, var_t* v, void *p);
-extern void new_object (exec_t* ex, var_t* v, void* p);
-extern void new_frame (exec_t* ex, var_t* v, void *p);
+extern int new_null (var_t* v);
+extern int new_bool(var_t* v, int i);
+extern int new_char (var_t* v, int i);
+extern int new_int (var_t* v, int i);
+extern int new_symbol (var_t* v, int i);
+extern int new_float (var_t* v, double f);
+extern int new_label (var_t* v, char *p);
+extern int new_function (var_t* v, char *p);
+extern int new_class (var_t* v, char *p);
+extern int new_string_with (exec_t* ex, var_t* v, char *s, int n);
+extern int new_string_copy (exec_t* ex, var_t* v, char *s, int n);
+extern int new_string (exec_t* ex, var_t* v, void *p);
+extern int new_array (exec_t* ex, var_t* v, void *p);
+extern int new_refer (exec_t* ex, var_t* v, void *p);
+extern int new_object (exec_t* ex, var_t* v, void* p);
+extern int new_frame (exec_t* ex, var_t* v, void *p);
 extern int serial_to_bytes(exec_t* ex, var_t* v, char **s, int *n);
 extern int unserial_from_bytes(exec_t* ex, var_t* v, char *s, int n);
 extern int echo_to_file(exec_t* ex, FILE* fp, char *s, int n);
@@ -198,7 +202,7 @@ extern int utime(int y, int m, int d, int h, int i, int s);
 extern void gdate(int t, int *y, int *m, int *d, int *h, int *i, int *s);
 extern int datetotime(int y, int m, int d, int h, int i, int s, int g);
 extern int strtotime(char *src, int len, int g);
-extern int timetostr(char dst[BUFSIZ], char *fmt, int n, int t, int g);
+extern int timetostr(char dst[BUFSIZ], char *fmt, int t, int g);
 extern int pack_to_bytes(exec_t* ex, char **dst, char* fmt, var_t* va, int vn);
 extern int unpack_from_bytes(exec_t* ex, char *fmt, char *src, int len, var_t* res);
 extern int strnmd5(char *d, char *s, unsigned long n);
@@ -216,10 +220,14 @@ extern int unentity(char **d, char *s, int n);
 extern int quote(char **d, char *s, int n, int q);
 extern int unquote(char **d, char *s, int n);
 extern int jcodeof(char *s, int n);
-extern int to_euc(char *d, char *s, int n);
-extern int to_sjis(char *d, char *s, int n);
-extern int to_jis(char *d, char *s, int n);
-extern int to_utf8(char *d, char *s, int n);
 extern int jconv(char **d, char *s, int n, int j);
+extern int jtrim(char **d, char *s, int n);
+extern int setupcgi(exec_t* ex, FILE* in);
+extern int flushcgi(exec_t* ex, FILE* out);
+extern void connect_timeout(int sig);
+extern int popmail(char *user, char *pass, FILE* out, FILE* err);
+extern int sendmail(char *to, char* data, FILE* err);
+extern int httpget(char *url, char *head, char* data, FILE* out, FILE* err);
+extern int ftpsite(char *user, char* pass, char *cmd, FILE* err);
 
 #endif /* _EXT_H_ */
