@@ -35,6 +35,7 @@
 #define K_echo        0x0CB8F46F
 #define K_else        0x0CBB39E5
 #define K_eval        0x0CBDB0EC
+#define K_exec        0x0CBE32E3
 #define K_exit        0x0CBE34F4
 #define K_tmpfile     0x0CD9D0BB
 #define K_fgid        0x0CD9F4E4
@@ -48,6 +49,7 @@
 #define K_splice      0x0D9A76DB
 #define K_replace     0x0D9BE7BB
 #define K_load        0x0D9BF0E4
+#define K_popmail     0x0DBBF212
 #define K_null        0x0DDD766C
 #define K_popen       0x0DFC32E0
 #define K_pack        0x0E1871EB
@@ -57,6 +59,7 @@
 #define K_rand        0x0E587764
 #define K_read        0x0E5970E4
 #define K_print       0x0E5A777A
+#define K_ftpsite     0x0E794D2B
 #define K_seek        0x0E7972EB
 #define K_sort        0x0E7BF974
 #define K_sqrt        0x0E7C7974
@@ -69,6 +72,7 @@
 #define K___LINE__    0x18AF9316
 #define K___FILE__    0x18EF93B6
 #define K_INFINITY    0x18F41E30
+#define K_setcookie   0x1A6D52D9
 #define K_isfinite    0x1C7DE408
 #define K_values      0x1D9D759F
 #define K_timegm      0x1DB974A0
@@ -113,7 +117,7 @@
 #define K_chown       0x3D1BFBE2
 #define K_class       0x3D9879FF
 #define K_close       0x3D9BF9E9
-#define K_pclose      0x3D9BFEE9
+#define K_putheader   0x3DEE947F
 #define K_count       0x3DFD7778
 #define K_split       0x3E1B34FA
 #define K_crypt       0x3E5E7878
@@ -123,6 +127,7 @@
 #define K_function    0x3F01D902
 #define K_M_LOG2E     0x49F3B4BC
 #define K_label       0x4C38B2E1
+#define K_sendmail    0x4C775A00
 #define K_stderr      0x4C997E4C
 #define K_stdout      0x4C9BFDCA
 #define K_continue    0x4CB4C40B
@@ -132,6 +137,7 @@
 #define K_log10       0x4DF9D8BD
 #define K_touch       0x4DFD71E6
 #define K_xmlopen     0x4DFFF433
+#define K_httpget     0x4E1AB5BA
 #define K_trace       0x4E5871EB
 #define K_untrace     0x4E5BDF0B
 #define K_toString    0x4F890A59
