@@ -42,6 +42,7 @@ case K_remove: *s = "remove"; return (*n = 6);
 case K_label: *s = "label"; return (*n = 5);
 case K_M_2_SQRTPI: *s = "M_2_SQRTPI"; return (*n = 10);
 case K_exit: *s = "exit"; return (*n = 4);
+case K_sendmail: *s = "sendmail"; return (*n = 8);
 case K_length: *s = "length"; return (*n = 6);
 case K_sprintf: *s = "sprintf"; return (*n = 7);
 case K_print: *s = "print"; return (*n = 5);
@@ -66,6 +67,7 @@ case K_trim: *s = "trim"; return (*n = 4);
 case K_rename: *s = "rename"; return (*n = 6);
 case K_atan2: *s = "atan2"; return (*n = 5);
 case K_time: *s = "time"; return (*n = 4);
+case K_putheader: *s = "putheader"; return (*n = 9);
 case K_return: *s = "return"; return (*n = 6);
 case K_floor: *s = "floor"; return (*n = 5);
 case K_if: *s = "if"; return (*n = 2);
@@ -73,6 +75,7 @@ case K_tracef: *s = "tracef"; return (*n = 6);
 case K_fmtime: *s = "fmtime"; return (*n = 6);
 case K_sqrt: *s = "sqrt"; return (*n = 4);
 case K_popen: *s = "popen"; return (*n = 5);
+case K_popmail: *s = "popmail"; return (*n = 7);
 case K_in: *s = "in"; return (*n = 2);
 case K_splice: *s = "splice"; return (*n = 6);
 case K_mkdir: *s = "mkdir"; return (*n = 5);
@@ -93,20 +96,23 @@ case K_sum: *s = "sum"; return (*n = 3);
 case K_this: *s = "this"; return (*n = 4);
 case K_tempnam: *s = "tempnam"; return (*n = 7);
 case K_atan: *s = "atan"; return (*n = 4);
+case K_ftpsite: *s = "ftpsite"; return (*n = 7);
 case K_join: *s = "join"; return (*n = 4);
 case K_try: *s = "try"; return (*n = 3);
 case K_cos: *s = "cos"; return (*n = 3);
 case K_symlink: *s = "symlink"; return (*n = 7);
-case K_chown: *s = "chown"; return (*n = 5);
 case K_pop: *s = "pop"; return (*n = 3);
+case K_chown: *s = "chown"; return (*n = 5);
 case K_decode: *s = "decode"; return (*n = 6);
 case K_max: *s = "max"; return (*n = 3);
 case K_keys: *s = "keys"; return (*n = 4);
 case K_xmlopen: *s = "xmlopen"; return (*n = 7);
+case K_setcookie: *s = "setcookie"; return (*n = 9);
 case K_pow: *s = "pow"; return (*n = 3);
 case K_setenv: *s = "setenv"; return (*n = 6);
 case K_min: *s = "min"; return (*n = 3);
 case K_rindex: *s = "rindex"; return (*n = 6);
+case K_exec: *s = "exec"; return (*n = 4);
 case K_toInt: *s = "toInt"; return (*n = 5);
 case K_fmode: *s = "fmode"; return (*n = 5);
 case K_sort: *s = "sort"; return (*n = 4);
@@ -119,15 +125,14 @@ case K_chmod: *s = "chmod"; return (*n = 5);
 case K_read: *s = "read"; return (*n = 4);
 case K_trace: *s = "trace"; return (*n = 5);
 case K_class: *s = "class"; return (*n = 5);
-case K_pclose: *s = "pclose"; return (*n = 6);
 case K_for: *s = "for"; return (*n = 3);
 case K_unpack: *s = "unpack"; return (*n = 6);
 case K_md5: *s = "md5"; return (*n = 3);
 case K_log10: *s = "log10"; return (*n = 5);
 case K_goto: *s = "goto"; return (*n = 4);
-case K_delete: *s = "delete"; return (*n = 6);
-case K_tell: *s = "tell"; return (*n = 4);
 case K_fctime: *s = "fctime"; return (*n = 6);
+case K_tell: *s = "tell"; return (*n = 4);
+case K_delete: *s = "delete"; return (*n = 6);
 case K_M_E: *s = "M_E"; return (*n = 3);
 case K_fopen: *s = "fopen"; return (*n = 5);
 case K_isInt: *s = "isInt"; return (*n = 5);
@@ -136,8 +141,8 @@ case K_isFloat: *s = "isFloat"; return (*n = 7);
 case K_function: *s = "function"; return (*n = 8);
 case K_isNumber: *s = "isNumber"; return (*n = 8);
 case K_fuid: *s = "fuid"; return (*n = 4);
-case K_push: *s = "push"; return (*n = 4);
 case K_println: *s = "println"; return (*n = 7);
+case K_push: *s = "push"; return (*n = 4);
 case K_writefile: *s = "writefile"; return (*n = 9);
 case K_do: *s = "do"; return (*n = 2);
 case K_unserial: *s = "unserial"; return (*n = 8);
@@ -166,6 +171,7 @@ case K_encode: *s = "encode"; return (*n = 6);
 case K_gmdate: *s = "gmdate"; return (*n = 6);
 case K_default: *s = "default"; return (*n = 7);
 case K_toString: *s = "toString"; return (*n = 8);
+case K_httpget: *s = "httpget"; return (*n = 7);
 case K_stderr: *s = "stderr"; return (*n = 6);
 case K___LINE__: *s = "__LINE__"; return (*n = 8);
 case K_flink: *s = "flink"; return (*n = 5);
