@@ -42,6 +42,7 @@ case K_remove: return K_remove;
 case K_label: return K_label;
 case K_M_2_SQRTPI: return K_M_2_SQRTPI;
 case K_exit: return K_exit;
+case K_sendmail: return K_sendmail;
 case K_length: return K_length;
 case K_sprintf: return K_sprintf;
 case K_print: return K_print;
@@ -66,6 +67,7 @@ case K_trim: return K_trim;
 case K_rename: return K_rename;
 case K_atan2: return K_atan2;
 case K_time: return K_time;
+case K_putheader: return K_putheader;
 case K_return: return K_return;
 case K_floor: return K_floor;
 case K_if: return K_if;
@@ -73,6 +75,7 @@ case K_tracef: return K_tracef;
 case K_fmtime: return K_fmtime;
 case K_sqrt: return K_sqrt;
 case K_popen: return K_popen;
+case K_popmail: return K_popmail;
 case K_in: return K_in;
 case K_splice: return K_splice;
 case K_mkdir: return K_mkdir;
@@ -93,20 +96,23 @@ case K_sum: return K_sum;
 case K_this: return K_this;
 case K_tempnam: return K_tempnam;
 case K_atan: return K_atan;
+case K_ftpsite: return K_ftpsite;
 case K_join: return K_join;
 case K_try: return K_try;
 case K_cos: return K_cos;
 case K_symlink: return K_symlink;
-case K_chown: return K_chown;
 case K_pop: return K_pop;
+case K_chown: return K_chown;
 case K_decode: return K_decode;
 case K_max: return K_max;
 case K_keys: return K_keys;
 case K_xmlopen: return K_xmlopen;
+case K_setcookie: return K_setcookie;
 case K_pow: return K_pow;
 case K_setenv: return K_setenv;
 case K_min: return K_min;
 case K_rindex: return K_rindex;
+case K_exec: return K_exec;
 case K_toInt: return K_toInt;
 case K_fmode: return K_fmode;
 case K_sort: return K_sort;
@@ -119,15 +125,14 @@ case K_chmod: return K_chmod;
 case K_read: return K_read;
 case K_trace: return K_trace;
 case K_class: return K_class;
-case K_pclose: return K_pclose;
 case K_for: return K_for;
 case K_unpack: return K_unpack;
 case K_md5: return K_md5;
 case K_log10: return K_log10;
 case K_goto: return K_goto;
-case K_delete: return K_delete;
-case K_tell: return K_tell;
 case K_fctime: return K_fctime;
+case K_tell: return K_tell;
+case K_delete: return K_delete;
 case K_M_E: return K_M_E;
 case K_fopen: return K_fopen;
 case K_isInt: return K_isInt;
@@ -136,8 +141,8 @@ case K_isFloat: return K_isFloat;
 case K_function: return K_function;
 case K_isNumber: return K_isNumber;
 case K_fuid: return K_fuid;
-case K_push: return K_push;
 case K_println: return K_println;
+case K_push: return K_push;
 case K_writefile: return K_writefile;
 case K_do: return K_do;
 case K_unserial: return K_unserial;
@@ -166,6 +171,7 @@ case K_encode: return K_encode;
 case K_gmdate: return K_gmdate;
 case K_default: return K_default;
 case K_toString: return K_toString;
+case K_httpget: return K_httpget;
 case K_stderr: return K_stderr;
 case K___LINE__: return K___LINE__;
 case K_flink: return K_flink;
