@@ -17,13 +17,14 @@
 #define LF_LEFT       0x20
 
 /* VALUE FLAG */
-#define VF_SYMBOL     0x10
-#define VF_REFER      0x20
-#define VF_UNRESOLV   0xF0
-#define VF_OBJECT     0x01
-#define VF_NEW        0x02
-#define VF_CONST      0x04
-#define VF_RESOLV     0x0F
+#define VF_OBJECT     0x0001
+#define VF_NEW        0x0002
+#define VF_CONST      0x0004
+#define VF_RETURN     0x0008
+#define VF_RESOLV     0x00FF
+#define VF_SYMBOL     0x0100
+#define VF_REFER      0x0200
+#define VF_UNRESOLV   0xFF00
 
 /* ALLOW FLAG */
 #define AF_BREAK      0x01
@@ -480,8 +481,8 @@ static int add_symbol(compile_t* cp, char *s, int n)
 	i = cp->nn++;
 	cp->np = realloc(cp->np, cp->nn * sizeof(cp_name_t));
 	cp->np[i].k = k;
-	cp->np[i].s = strndupok(s, n);
-	cp->np[i].n = n;
+	cp->np[i].s = calloc(n + 1, 1);
+	memcpy(cp->np[i].s, s, cp->np[i].n = n);
 	return k;
 }
 static int cp_symbol_to_string(compile_t* cp, int k, char **s, int *n)
@@ -543,8 +544,8 @@ static void add_define(compile_t* cp, int k, char* s, int n)
 		free(cp->dp[i].s);
 	}
 	cp->dp[i].k = k;
-	cp->dp[i].s = strndupok(s, n);
-	cp->dp[i].n = n;
+	cp->dp[i].s = calloc(n + 1, 1);
+	memcpy(cp->dp[i].s, s, cp->dp[i].n = n);
 }
 static void add_local(compile_t* cp, op_t c, int k, int j)
 {
@@ -646,20 +647,6 @@ static int is_typecast(op_t c)
 /************************************************************
  *
  ************************************************************/
-static int read_as_string(char *p, char **s)
-{
-	switch (*p++) {
-	case C_null:  *s = "null" ; return 4;
-	case C_false: *s = "false"; return 5;
-	case C_true:  *s = "true" ; return 4;
-	case C_char:  return sprintf(*s = tmpbuf(), "%c", *(int*)p);
-	case C_int:   return sprintf(*s = tmpbuf(), "%d", *(int*)p);
-	case C_float: return sprintf(*s = tmpbuf(), "%g", *(double*)p);
-	case C_string: *s = p + 4; return *(int*)p;
-	}
-	*s = "";
-	return 0;
-}
 static double read_as_float(char *p)
 {
 	switch (*p++) {
@@ -686,9 +673,24 @@ static int read_as_int(char *p)
 	}
 	return 0;
 }
+static int read_as_string(char *p, char **s)
+{
+	switch (*p++) {
+	case C_null:  *s = "null" ; return 4;
+	case C_false: *s = "false"; return 5;
+	case C_true:  *s = "true" ; return 4;
+	case C_char:  return sprintf(*s, "%c", *(int*)p);
+	case C_int:   return sprintf(*s, "%d", *(int*)p);
+	case C_float: return sprintf(*s, "%g", *(double*)p);
+	case C_string: *s = p + 4; return *(int*)p;
+	}
+	*s = "";
+	return 0;
+}
 static void calc_const(compile_t* cp, int i1, op_t c, int i2)
 {
-	char *p1, *p2, *s1, *s2;
+	static char pk1[32], pk2[32];
+	char *p1, *p2, *s1=pk1, *s2=pk2;
 	double f1, f2;
 	int n1, n2;
 	p1 = cp->bc + i1;
@@ -1067,6 +1069,59 @@ static int expr_left(compile_t* cp)
 		c = expr_post(cp);
 	} else if (c == C_symbol) {
 		c = expr_symbol(cp);
+	} else if (c == C_break) {
+		exerrif(cp, !(cp->fp->af & AF_BREAK), "bad break");
+		put_c_int(cp, C_goto, K_break);
+		c = lex(cp, 0);
+		exerrif(cp, c != ';', "bad break");
+		cp->vf = VF_RETURN;
+	} else if (c == C_continue) {
+		exerrif(cp, !(cp->fp->af & AF_CONTINUE), "bad continue");
+		put_c_int(cp, C_goto, K_continue);
+		c = lex(cp, 0);
+		exerrif(cp, c != ';', "bad continue");
+		cp->vf = VF_RETURN;
+	} else if (c == C_return) {
+		exerrif(cp, !(cp->fp->af & AF_RETURN), "bad return");
+		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad return");
+		if (c == ';') {
+			put_c(cp, C_null);
+		} else {
+			unlex(cp, c);
+			c = expr_left(cp);
+		}
+		exerrif(cp, c != ';', "bad return");
+		put_c(cp, C_return);
+		cp->vf = VF_RETURN;
+	} else if (c == C_throw) {
+		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad throw");
+		if (c == ';') {
+			put_c(cp, C_null);
+		} else {
+			unlex(cp, c);
+			c = expr_left(cp);
+		}
+		exerrif(cp, c != ';', "bad throw");
+		put_c(cp, C_throw);
+		cp->vf = VF_RETURN;
+	} else if (c == C_exit) {
+		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad exit");
+		if (c == ';') {
+			put_c(cp, C_null);
+		} else {
+			unlex(cp, c);
+			c = expr_left(cp);
+		}
+		exerrif(cp, c != ';', "bad exit");
+		put_c(cp, C_exit);
+		cp->vf = VF_RETURN;
+	} else if (c == C_goto) {
+		c = lex(cp, 0);
+		exerrif(cp, c != C_symbol, "bad label");
+		put_c_int(cp, C_goto, add_symbol(cp, cp->sc, cp->nc));
+		c = lex(cp, 0);
+		exerrif(cp, c != ';', "bad goto");
+		cp->vf = VF_RETURN;
 	}
 	return c;
 }
@@ -1079,7 +1134,7 @@ static int expr_next(compile_t* cp, int prev)
 	j = cp->bn;
 	c = expr_left(cp);
 	vf = cp->vf;
-	if (!vf) {
+	if (!vf || (vf == VF_RETURN)) {
 		return c;
 	}
 	else if (c == '=') {
@@ -1176,6 +1231,33 @@ static int expr_part(compile_t* cp)
 	return c;
 }
 
+static int expr_if(compile_t* cp)
+{
+	op_t c;
+	int k;
+	put_c(cp, C_line);
+	put_ushort(cp, cp->ln);
+	exerrif(cp, (c = lex(cp, 0)) <= 1, "bad for expr");
+	if (c == C_var) {
+		c = lex(cp, 0);
+		exerrif(cp, c != C_symbol, "bad define var");
+		k = add_symbol(cp, cp->sc, cp->nc);
+		add_local(cp, C_var, k, 0);
+		c = lex(cp, 0);
+		exerrif(cp, c != '=', "bad assign local");
+		put_c_int(cp, C_symbol, k);
+		c = expr_next(cp, 0);
+		exerrif(cp, !(cp->vf & VF_RESOLV), "bad expr");
+		put_c(cp, C_store);
+		put_c(cp, C_load);
+	} else {
+		unlex(cp, c);
+		c = expr_next(cp, 0);
+		exerrif(cp, !(cp->vf & VF_RESOLV), "bad expr");
+	}
+	return c;
+}
+
 static int expr_var(compile_t* cp)
 {
 	op_t c;
@@ -1344,10 +1426,15 @@ static int stmt_Minclude(compile_t* cp)
 		return ';';
 	}
 	
-	if (!(sc = vmopen(fn, &sz, &fp))) {
-		fprintf(cp->err, "vmopen error: %s %s\n", fn, strerror(errno));
+	if (!(fp = fopen(fn, "r"))) {
+		fprintf(cp->err, "fopen error: %s %s\n", fn, strerror(errno));
 		return 0;
 	}
+	fseek(fp, 0, SEEK_END);
+	sz = ftell(fp);
+	rewind(fp);
+	sc = mmap(0, sz, PROT_READ, MAP_SHARED, fileno(fp), 0);
+	assert(sc && (sc != MAP_FAILED));
 	
 	memcpy(&old, cp, sizeof(compile_t));
 	memset(cp, 0, sizeof(compile_t));
@@ -1369,7 +1456,9 @@ static int stmt_Minclude(compile_t* cp)
 	
 	c = stmt_file(cp, NULL);
 	
-	vmclose(sc, sz, fp);
+	// vmclose(sc, sz, fp);
+	assert( munmap(sc, sz) == 0 );
+	assert( fclose(fp) == 0 );
 	
 	old.np = cp->np;
 	old.nn = cp->nn;
@@ -1385,6 +1474,7 @@ static int stmt_Minclude(compile_t* cp)
 	
 	return ';';
 }
+
 static int stmt_Mdefine(compile_t* cp)
 {
 	op_t c;
@@ -1417,7 +1507,7 @@ static int stmt_if(compile_t* cp)
 	
 	exerrif(cp, c != '(', "bad if expr");
 	
-	c = expr_part(cp);
+	c = expr_if(cp);
 	
 	exerrif(cp, c != ')', "bad if expr");
 	
@@ -1578,9 +1668,9 @@ static int stmt_while(compile_t* cp)
 	c = lex(cp, 0);
 	exerrif(cp, c != '(', "bad while expr");
 	
-	c = expr_part(cp);
+	c = expr_if(cp);
 	
-	exerrif(cp, c != ')', "bad whiel expr");
+	exerrif(cp, c != ')', "bad while expr");
 	
 	put_c(cp, C_loop);
 	e = seek_int(cp, 0);
@@ -1617,7 +1707,7 @@ static int stmt_do(compile_t* cp)
 	
 	add_local(cp, C_label, K_continue, cp->bn);
 	
-	c = expr_part(cp);
+	c = expr_if(cp);
 	
 	exerrif(cp, c != ')', "bad while expr");
 	
@@ -1647,7 +1737,7 @@ static int stmt_switch(compile_t* cp)
 	c = lex(cp, 0);
 	exerrif(cp, c != '(', "bad switch expr");
 	
-	c = expr_part(cp);
+	c = expr_if(cp);
 	
 	exerrif(cp, c != ')', "bad switch expr");
 	
@@ -1840,6 +1930,7 @@ static int stmt_class(compile_t* cp)
 	
 	return ';';
 }
+
 static int stmt_block(compile_t* cp)
 {
 	cp_frame_t fp;
@@ -1866,6 +1957,7 @@ static int stmt_block(compile_t* cp)
 	
 	return ';';
 }
+
 static int stmt_line(compile_t* cp)
 {
 	op_t c;
@@ -1895,53 +1987,6 @@ static int stmt_line(compile_t* cp)
 	} else if (c == C_function) {
 		exerrif(cp, !(cp->fp->af & AF_FUNCTION), "bad function");
 		c = stmt_function(cp);
-	} else if (c == C_break) {
-		exerrif(cp, !(cp->fp->af & AF_BREAK), "bad break");
-		put_c_int(cp, C_goto, K_break);
-		c = lex(cp, 0);
-		exerrif(cp, c != ';', "bad break");
-	} else if (c == C_continue) {
-		exerrif(cp, !(cp->fp->af & AF_CONTINUE), "bad continue");
-		put_c_int(cp, C_goto, K_continue);
-		c = lex(cp, 0);
-		exerrif(cp, c != ';', "bad continue");
-	} else if (c == C_return) {
-		exerrif(cp, !(cp->fp->af & AF_RETURN), "bad return");
-		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad return");
-		if (c == ';') {
-			put_c(cp, C_null);
-		} else {
-			unlex(cp, c);
-			c = expr_part(cp);
-		}
-		exerrif(cp, c != ';', "bad return");
-		put_c(cp, C_return);
-	} else if (c == C_throw) {
-		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad throw");
-		if (c == ';') {
-			put_c(cp, C_null);
-		} else {
-			unlex(cp, c);
-			c = expr_part(cp);
-		}
-		exerrif(cp, c != ';', "bad throw");
-		put_c(cp, C_throw);
-	} else if (c == C_exit) {
-		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad exit");
-		if (c == ';') {
-			put_c(cp, C_null);
-		} else {
-			unlex(cp, c);
-			c = expr_part(cp);
-		}
-		exerrif(cp, c != ';', "bad exit");
-		put_c(cp, C_exit);
-	} else if (c == C_goto) {
-		c = lex(cp, 0);
-		exerrif(cp, c != C_symbol, "bad label");
-		put_c_int(cp, C_goto, add_symbol(cp, cp->sc, cp->nc));
-		c = lex(cp, 0);
-		exerrif(cp, c != ';', "bad goto");
 	} else if (c == C_Minclude) {
 		c = stmt_Minclude(cp);
 	} else if (c == C_Mdefine) {
@@ -1969,20 +2014,18 @@ static int stmt_line(compile_t* cp)
  ************************************************************/
 static int stmt_file(compile_t* cp, cp_frame_t* fp)
 {
-	char *sc;
 	op_t c;
 	int sz;
 	
-	sc = cp->sc;
-	if (*sc=='#' && *(sc + 1)=='!') {
-		while (*sc && !iscrlf(*sc)) sc++;
+	if (*cp->sc=='#' && *(cp->sc + 1)=='!') {
+		while (*cp->sc && !iscrlf(*cp->sc)) cp->sc++;
 	}
-	while (isspace(*sc)) sc++;
-	sz = cp->es - sc;
+	while (isspace(*cp->sc)) cp->sc++;
+	sz = cp->es - cp->sc;
 	
-	if (*sc == 1) {
+	if (*cp->sc == 1) {
 		cp->bc = realloc(cp->bc, cp->bn + sz + 1);
-		memcpy(cp->bc + cp->bn, sc, sz);
+		memcpy(cp->bc + cp->bn, cp->sc, sz);
 		cp->bn += sz;
 		return 1;
 	}
@@ -1999,7 +2042,7 @@ static int stmt_file(compile_t* cp, cp_frame_t* fp)
 			| AF_FILE | AF_BLOCK);
 	}
 	
-	if (*sc == '<') {
+	if (*cp->sc == '<') {
 		seek_doc(cp);
 		put_doc(cp, cp->sc, cp->nc, 0);
 	}
@@ -2065,14 +2108,21 @@ int compile_file(exec_t* ex, char *fn, char **bc, int* bn)
 	char* sc;
 	int sz, r;
 	
-	if (!(sc = vmopen(fn, &sz, &fp))) {
-		fprintf(ex->err, "vmopen error: %s %s\n", fn, strerror(errno));
-		return -1;
+	if (!(fp = fopen(fn, "r"))) {
+		fprintf(ex->err, "fopen error: %s %s\n", fn, strerror(errno));
+		return 0;
 	}
+	fseek(fp, 0, SEEK_END);
+	sz = ftell(fp);
+	rewind(fp);
+	sc = mmap(0, sz, PROT_READ, MAP_SHARED, fileno(fp), 0);
+	assert(sc && (sc != MAP_FAILED));
 	
 	r = compile_code(ex, fn, sc, sz, bc, bn);
 	
-	vmclose(sc, sz, fp);
+	// vmclose(sc, sz, fp);
+	assert( munmap(sc, sz) == 0 );
+	assert( fclose(fp) == 0 );
 	
 	return r;
 }
