@@ -7,6 +7,7 @@
 #include "_ext.h"
 
 typedef struct DB_t  DB_t;
+typedef struct DBM_t DBM_t;
 typedef struct DBI_t DBI_t;
 
 struct DBI_t {
@@ -18,36 +19,42 @@ struct DBI_t {
 	size_t data;
 };
 
-struct DB_t {
+struct DBM_t {	
+	size_t cmpi;
+	size_t seqi;
+	
+	size_t tail;
+	size_t dump;
+	size_t entry;
+	
 	size_t top;
 	size_t bot;
 	size_t min;
 	size_t max;
+};
+
+struct DB_t {
+	DBM_t* mp;
 	FILE* fp;
 	size_t sz;
-	size_t cmpi;
-	size_t seqi;
-	int (*cmpp)(DB_t*, size_t, char*, int);
-	int (*seqp)(DB_t*, size_t);
-	size_t entry;
-	size_t dump;
-	size_t tail;
+	int (*cmpp)(DBM_t*, size_t, char*, int);
+	int (*seqp)(DBM_t*, size_t);
 };
 
 /************************************************************
  *
  ************************************************************/
-static void* dballoc(DB_t* db, size_t sz)
+static void* dballoc(DBM_t* mp, size_t sz)
 {
 	size_t old, off, prev, next;
 	
 	if (sz < 4) sz = 4;
 	
-	if (!db->dump) {
-		db->dump = db->tail;
-		memset((void*)db + db->dump, 0, 8);
+	if (!mp->dump) {
+		mp->dump = mp->tail;
+		memset((void*)mp + mp->dump, 0, 8);
 	}
-	off = db->dump;
+	off = mp->dump;
 	prev = next = 0;
 	
 	do {
@@ -55,86 +62,85 @@ static void* dballoc(DB_t* db, size_t sz)
 			prev = off;
 			off = next;
 		}
-		old  = *((size_t*)((void*)db + off));
-		next = *((size_t*)((void*)db + off + 4));
+		old  = *((size_t*)((void*)mp + off));
+		next = *((size_t*)((void*)mp + off + 4));
 	} while (old && old < sz);
 	
 	if (!next) {
-		assert((old == 0) && (off == db->tail));
-		*((size_t*)((void*)db + off)) = sz;
-		db->tail = next = off + 4 + sz;
+		assert((old == 0) && (off == mp->tail));
+		*((size_t*)((void*)mp + off)) = sz;
+		mp->tail = next = off + 4 + sz;
 	} else {
-		assert((old && old >= sz) && (off != db->tail));
+		assert((old && old >= sz) && (off != mp->tail));
 	}
 	if (!prev) {
-		db->dump = next;
+		mp->dump = next;
 	} else {
-		*((size_t*)((void*)db + prev + 4)) = next;
+		*((size_t*)((void*)mp + prev + 4)) = next;
 	}
 	
-	return ((void*)db + off + 4);
+	return ((void*)mp + off + 4);
 }
-static void dbfree(DB_t* db, void *p)
+static void dbfree(DBM_t* mp, void *p)
 {
-	*((size_t*)p) = db->dump;
-	db->dump = (p - 4) - (void*)db;
+	*((size_t*)p) = mp->dump;
+	mp->dump = (p - 4) - (void*)mp;
 }
-
-static void dbread(DB_t* db, size_t off, char **data, int *len)
+static void dbread(DBM_t* mp, size_t off, char **data, int *len)
 {
-	void *p = (void*)db + off;
+	void *p = (void*)mp + off;
 	if (len) *len = *((int*)p);
 	if (data) *data = p + 4;
 }
-static size_t dbwrite(DB_t* db, char *data, int len)
+static size_t dbwrite(DBM_t* mp, char *data, int len)
 {
-	void *p = dballoc(db, 4 + len + 1);
+	void *p = dballoc(mp, 4 + len + 1);
 	*((int*)p) = len;
 	/* strncpy */
 	memcpy(p + 4, data, len);
 	*(char*)(p + 4 + len) = 0;
-	return ((void*)p - (void*)db);
+	return ((void*)p - (void*)mp);
 }
 
 /************************************************************
  *
  ************************************************************/
-static int dbseq_0(DB_t* db, size_t off)
+static int dbseq_0(DBM_t* mp, size_t off)
 {
-	return (!off ? db->top : ((DBI_t*)((void*)db + off))->dn);
+	return (!off ? mp->top : ((DBI_t*)((void*)mp + off))->dn);
 }
-static int dbseq_1(DB_t* db, size_t off)
+static int dbseq_1(DBM_t* mp, size_t off)
 {
-	return (!off ? db->bot : ((DBI_t*)((void*)db + off))->up);
+	return (!off ? mp->bot : ((DBI_t*)((void*)mp + off))->up);
 }
-static int dbseq_2(DB_t* db, size_t off)
+static int dbseq_2(DBM_t* mp, size_t off)
 {
-	return (!off ? db->min : ((DBI_t*)((void*)db + off))->gt);
+	return (!off ? mp->min : ((DBI_t*)((void*)mp + off))->gt);
 }
-static int dbseq_3(DB_t* db, size_t off)
+static int dbseq_3(DBM_t* mp, size_t off)
 {
-	return (!off ? db->max : ((DBI_t*)((void*)db + off))->lt);
+	return (!off ? mp->max : ((DBI_t*)((void*)mp + off))->lt);
 }
 
 static int dbreset(DB_t* db)
 {
-	if (db->seqi == 0) {
+	if (db->mp->seqi == 0) {
 		return 0;
 	}
-	db->seqi = 0;
+	db->mp->seqi = 0;
 	return 1;
 }
 static int dbseq(DB_t* db, char** name, int *nlen, char** data, int *dlen)
 {
-	DBI_t* di;
+	DBI_t* ip;
 	
-	if (!(db->seqi = (*db->seqp)(db, db->seqi))) {
+	if (!(db->mp->seqi = (*db->seqp)(db->mp, db->mp->seqi))) {
 		return 0;
 	}
 	
-	di = (void*)db + db->seqi;
-	dbread(db, di->name, name, nlen);
-	dbread(db, di->data, data, dlen);
+	ip = (void*)db->mp + db->mp->seqi;
+	dbread(db->mp, ip->name, name, nlen);
+	dbread(db->mp, ip->data, data, dlen);
 	
 	return 1;
 }
@@ -142,42 +148,42 @@ static int dbseq(DB_t* db, char** name, int *nlen, char** data, int *dlen)
 /************************************************************
  *
  ************************************************************/
-static int dbcmp_0(DB_t* db, size_t off, char* s2, int n2)
+static int dbcmp_0(DBM_t* mp, size_t off, char* s2, int n2)
 {
 	char *s1;
 	int n1,i,r;
-	dbread(db, off, &s1, &n1);
+	dbread(mp, off, &s1, &n1);
 	for (i=0; (i < n1) && (i < n2); i++) {
 		if ((r = s1[i] - s2[i]) != 0) return r;
 	}
 	return (n1 - n2);
 }
-static int dbcmp_1(DB_t* db, size_t off, char* s2, int n2)
+static int dbcmp_1(DBM_t* mp, size_t off, char* s2, int n2)
 {
 	char *s1;
 	int n1,i,r;
-	dbread(db, off, &s1, &n1);
+	dbread(mp, off, &s1, &n1);
 	if ((r = n1 - n2) != 0) return r;
 	for (i=0; i < n1; i++) {
 		if ((r = s1[i] - s2[i]) != 0) return r;
 	}
 	return 0;
 }
-static int dbcmp_2(DB_t* db, size_t off, char* s2, int n2)
+static int dbcmp_2(DBM_t* mp, size_t off, char* s2, int n2)
 {
 	char *s1;
 	int n1,i,r;
-	dbread(db, off, &s1, &n1);
+	dbread(mp, off, &s1, &n1);
 	for (i=0; (i < n1) && (i < n2); i++) {
 		if ((r = tolower(s1[i]) - tolower(s2[i])) != 0) return r;
 	}
 	return (n1 - n2);
 }
-static int dbcmp_3(DB_t* db, size_t off, char* s2, int n2)
+static int dbcmp_3(DBM_t* mp, size_t off, char* s2, int n2)
 {
 	char *s1;
 	int n1,i,r;
-	dbread(db, off, &s1, &n1);
+	dbread(mp, off, &s1, &n1);
 	if ((r = n1 - n2) != 0) return r;
 	for (i=0; i < n1; i++) {
 		if ((r = tolower(s1[i]) - tolower(s2[i])) != 0) return r;
@@ -190,119 +196,120 @@ static int dbcmp_3(DB_t* db, size_t off, char* s2, int n2)
  ************************************************************/
 static int dbget(DB_t* db, char *name, int nlen, char** data, int *dlen)
 {
-	DBI_t* di;
+	DBI_t* ip;
 	size_t off;
 	int r;
-	off = db->entry;
+	off = db->mp->entry;
 	while (off) {
-		di = (void*)db + off;
-		r = (*db->cmpp)(db, di->name, name, nlen);
+		ip = (void*)db->mp + off;
+		r = (*db->cmpp)(db->mp, ip->name, name, nlen);
 		if (r == 0) {
-			dbread(db, di->data, data, dlen);
+			dbread(db->mp, ip->data, data, dlen);
 			return 1;
 		} else if (r < 0) {
-			off = di->gt;
+			off = ip->gt;
 		} else {
-			off = di->lt;
+			off = ip->lt;
 		}
 	}
 	return 0;
 }
 static int dbset(DB_t* db, char *name, int nlen, char *data, int dlen)
 {
-	DBI_t *di;
+	DBI_t *ip;
 	size_t off, gt, lt;
 	int r;
-	off = db->entry;
+	off = db->mp->entry;
 	gt = lt = 0;
 	while (off) {
-		di = (void*)db + off;
-		r = (*db->cmpp)(db, di->name, name, nlen);
+		ip = (void*)db->mp + off;
+		r = (*db->cmpp)(db->mp, ip->name, name, nlen);
 		if (!r) {
 			break;
 		} else if (r < 0) {
 			lt = off;
-			off = gt ? 0 : di->gt;
+			off = gt ? 0 : ip->gt;
 		} else {
 			gt = off;
-			off = lt ? 0 : di->lt;
+			off = lt ? 0 : ip->lt;
 		}
 	}
 	if (!off) {
-		di = dballoc(db, sizeof(DBI_t));
-		off = (void*)di - (void*)db;
-		di->lt = lt;
-		di->gt = gt;
+		ip = dballoc(db->mp, sizeof(DBI_t));
+		off = (void*)ip - (void*)db->mp;
+		ip->lt = lt;
+		ip->gt = gt;
 		
-		if (!db->entry) db->entry = off;
+		if (!db->mp->entry) db->mp->entry = off;
 		
-		if (gt) ((DBI_t*)((void*)db + gt))->lt = off;
-		else db->max = off;
-		if (lt) ((DBI_t*)((void*)db + lt))->gt = off;
-		else db->min = off;
+		if (gt) ((DBI_t*)((void*)db->mp + gt))->lt = off;
+		else db->mp->max = off;
+		if (lt) ((DBI_t*)((void*)db->mp + lt))->gt = off;
+		else db->mp->min = off;
 		
-		di->up = db->bot;
-		di->dn = 0;
-		if (di->up) ((DBI_t*)(size_t*)((void*)db + di->up))->dn = off;
-		else db->top = off;
-		db->bot = off;
+		ip->up = db->mp->bot;
+		ip->dn = 0;
+		if (ip->up) ((DBI_t*)(size_t*)((void*)db->mp + ip->up))->dn = off;
+		else db->mp->top = off;
+		db->mp->bot = off;
 		
-		di->name = dbwrite(db, name, nlen);
+		ip->name = dbwrite(db->mp, name, nlen);
 	} else {
-		dbfree(db, (void*)db + di->data);
+		dbfree(db->mp, (void*)db->mp + ip->data);
 	}
-	di->data = dbwrite(db, data, dlen);
+	ip->data = dbwrite(db->mp, data, dlen);
 	return 1;
 }
 static int dbdel(DB_t* db, char *name, int nlen)
 {
-	DBI_t *di;
+	DBI_t *ip;
 	size_t off;
 	int r;
-	off = db->entry;
+	off = db->mp->entry;
 	while (off) {
-		di = (void*)db + off;
-		r = (*db->cmpp)(db, di->name, name, nlen);
+		ip = (void*)db->mp + off;
+		r = (*db->cmpp)(db->mp, ip->name, name, nlen);
 		if (!r) {
 			break;
 		} else if (r < 0) {
-			off = di->gt;
+			off = ip->gt;
 		} else {
-			off = di->lt;
+			off = ip->lt;
 		}
 	}
 	if (!off) {
 		return 0;
 	}
 	
-	if (off == db->entry) db->entry = di->lt ? di->lt : di->gt;
+	if (off == db->mp->entry) db->mp->entry = ip->lt ? ip->lt : ip->gt;
 	
-	if (di->gt) ((DBI_t*)((void*)db + di->gt))->lt = di->lt;
-	else db->max  = di->lt;
-	if (di->lt) ((DBI_t*)((void*)db + di->lt))->gt = di->gt;
-	else db->min = di->gt;
+	if (ip->gt) ((DBI_t*)((void*)db->mp + ip->gt))->lt = ip->lt;
+	else db->mp->max  = ip->lt;
+	if (ip->lt) ((DBI_t*)((void*)db->mp + ip->lt))->gt = ip->gt;
+	else db->mp->min = ip->gt;
 	
-	if (di->up) ((DBI_t*)((void*)db + di->up))->dn = di->dn;
-	else db->top = di->dn;
-	if (di->dn) ((DBI_t*)((void*)db + di->dn))->up = di->up;
-	else db->bot = di->up;
+	if (ip->up) ((DBI_t*)((void*)db->mp + ip->up))->dn = ip->dn;
+	else db->mp->top = ip->dn;
+	if (ip->dn) ((DBI_t*)((void*)db->mp + ip->dn))->up = ip->up;
+	else db->mp->bot = ip->up;
 	
-	dbfree(db, (void*)db + di->data);
-	dbfree(db, (void*)db + di->name);
-	dbfree(db, di);
+	dbfree(db->mp, (void*)db->mp + ip->data);
+	dbfree(db->mp, (void*)db->mp + ip->name);
+	dbfree(db->mp, ip);
 	return 1;
 }
+
 /************************************************************
  *
  ************************************************************/
 static int dbtrace(DB_t* db, FILE* fp)
 {
-	size_t off = db->dump;
+	size_t off = db->mp->dump;
 	int i = 0;
 	while (off) {
 		fprintf(fp, "%d: %d: %d\n",
-			i++, (int)off, (int)*((size_t*)((void*)db + off)));
-		off = *((size_t*)((void*)db + off + 4));
+			i++, (int)off, (int)*((size_t*)((void*)db->mp + off)));
+		off = *((size_t*)((void*)db->mp + off + 4));
 	}
 	return i;
 }
@@ -310,50 +317,53 @@ static int dblength(DB_t* db)
 {
 	size_t off;
 	int len = 0;
-	off = db->min;
+	off = db->mp->min;
 	while (off) {
-		off = ((DBI_t*)((void*)db + off))->gt;
+		off = ((DBI_t*)((void*)db->mp + off))->gt;
 		len++;
 	}
 	return len;
 }
 static int dbsync(DB_t* db)
 {
-	size_t sz;
 	int r;
-	sz = db->sz;
-	r = msync(db, sz, MS_SYNC) == 0;
+	r = msync(db->mp, db->sz, MS_SYNC) == 0;
 	return r;
 }
 static int dbclose(DB_t* db)
 {
-	FILE* fp;
-	size_t sz;
 	int r1,r2,r3;
 	
-	if (0) {
-		dblength(db);
-		dbsync(db);
+	r1 = r2 = r3 = 0;
+	
+	if (db->mp) {
+		if (0) {
+			dblength(db);
+			dbsync(db);
+		}
+		
+		r1 = msync(db->mp, db->sz, MS_SYNC) == 0;
+		r2 = munmap(db->mp, db->sz) == 0;
+		r3 = fclose(db->fp) == 0;
+		
+		memset(db, 0, sizeof(DB_t));
 	}
 	
-	fp = db->fp;
-	sz = db->sz;
-	r1 = msync(db, sz, MS_SYNC) == 0;
-	r2 = munmap(db, sz) == 0;
-	r3 = fclose(fp) == 0;
 	return (r1 && r2 && r3);
 }
 
 /************************************************************
  *
  ************************************************************/
-static DB_t* dbopen(char *path, char* mode)
+static int dbopen(DB_t* db, char *path, char* mode)
 {
 	FILE* fp;
-	DB_t* db;
+	DBM_t* mp;
 	size_t sz;
 	int ex, cmp, seq;
 	
+	memset(db, 0, sizeof(DB_t));
+	
 	sz = 0x10000;
 	cmp = seq = 0;
 	if (mode) {
@@ -386,18 +396,19 @@ static DB_t* dbopen(char *path, char* mode)
 	}
 	rewind(fp);
 	
-	db = mmap(0, sz, PROT_READ|PROT_WRITE, MAP_SHARED, fileno(fp), 0);
-	assert(db && (db != MAP_FAILED));
+	mp = mmap(0, sz, PROT_READ|PROT_WRITE, MAP_SHARED, fileno(fp), 0);
+	assert(mp && (mp != MAP_FAILED));
 	
 	if (!ex) {
-		memset(db, 0, sizeof(DB_t));
-		db->tail = sizeof(DB_t);
-		db->cmpi = cmp;
+		memset(mp, 0, sizeof(DBM_t));
+		mp->tail = sizeof(DBM_t);
+		mp->cmpi = cmp;
 	} else {
-		cmp = db->cmpi;
-		db->seqi = 0;
+		cmp = mp->cmpi;
+		mp->seqi = 0;
 	}
 	
+	db->mp = mp;
 	db->fp = fp;
 	db->sz = sz;
 	
@@ -415,18 +426,20 @@ static DB_t* dbopen(char *path, char* mode)
 	case 3: db->seqp = dbseq_3; break;
 	}
 	
-	return db;
+	return 1;
 }
 
 /************************************************************
  *
  ************************************************************/
-int dbm_open(exec_t* ex, var_t* v, char* path, char* mode)
+int new_dbm(exec_t* ex, var_t* v, char* path, char* mode)
 {
 	DB_t *db;
-	db = dbopen(path, mode);
-	if (!db) {
+	v->p = NULL;
+	db = (DB_t*)alloc_link(ex, sizeof(DB_t), dbm_proto);
+	if (!dbopen(db, path, mode)) {
 		fprintf(ex->err, "fopen error: %s %s\n", path, strerror(errno));
+		free_link(ex, (void*)db);
 		return 0;
 	}
 	v->p = dbm_proto;
@@ -434,6 +447,18 @@ int dbm_open(exec_t* ex, var_t* v, char* path, char* mode)
 	return 1;
 }
 
+/************************************************************
+ *
+ ************************************************************/
+void dbm_clear(exec_t* ex, void *p)
+{
+	dbclose(p);
+}
+void dbm_trace(exec_t* ex, void *p)
+{
+	dbtrace(p, ex->out);
+}
+
 /************************************************************
  *
  ************************************************************/
@@ -539,4 +564,3 @@ int dbm_callFunc(exec_t* ex, void* p, int sym, int argc, var_t* argv, var_t* res
 	}
 	return -1;
 }
-
