@@ -180,16 +180,13 @@ int url_decode(char *d, char *s, int n)
 /************************************************************
  *
  ************************************************************/
-extern int to_euc(char *d, char *s, int n);
-extern int to_sjis(char *d, char *s, int n);
-extern int to_jis(char *d, char *s, int n);
-extern int to_utf8(char *d, char *s, int n);
+extern int jconv(char **d, char *s, int n, int j);
 
 static int mime_encode(char *d, char *s, int n)
 {
 	char *div = ",=:;";
 	char *e, *s2, *e2;
-	int c, m;
+	int m;
 	
 	e = s + n;
 	s2 = s;
@@ -200,43 +197,54 @@ static int mime_encode(char *d, char *s, int n)
 			continue;
 		}
 		e2 = s2;
-		c = 0;
+		m = 0;
 		if (*e2++ == '"') {
 			while ((e2 < e) && *e2 && (*e2 != '"')) {
 				if (*e2 == '\\') e2++;
-				c += !isprint(*e2++);
+				m += !isprint(*e2++);
 			}
 		} else {
 			while ((e2 < e) && *e2
 			&& !isspace(*e2) && !strchr(div, *e2)) {
-				c += !isprint(*e2++);
+				m += !isprint(*e2++);
 			}
 		}
-		if (!c) {
+		if (!m) {
 			s2 = e2;
 			continue;
 		}
 		if ((m = s2 - s) > 0) {
-			n += sputsn(d+n, s, m);
+			if (d) memcpy(d + n, s, m);
+			n += m;
+		}
+		if (*s2 == '"') {
+			if (d) d[n]=*s2++;
+			n++;
 		}
-		if (*s2=='"') d[n++]=*s2++;
 		if ((m = e2 - s2) < 2) {
 			;
-		} else {
-			s = malloc((m * 4) + 1);
-			m = to_jis(s, s2, m);
-			n += sputs(d+n, "=?iso-2022-jp?b?");
+		} else if (d) {
+			m = jconv(&s, s2, m, 'j');
+			strcpy(d + n, "=?iso-2022-jp?b?");
+			n += 16;
 			n += base64_encode(d+n, s, m);
-			n += sputs(d+n, "?=");
+			strcpy(d + n, "?=");
+			n += 2;
 			free(s);
+		} else {
+			n += (m * 8) + 18;
+		}
+		if (*e2=='"') {
+			if (d) d[n]=*e2++;
+			n++;
 		}
-		if (*e2=='"') d[n++]=*e2++;
 		s = s2 = e2;
 	}
 	if ((m = e - s) > 0) {
-		n += sputsn(d+n, s, m);
+		if (d) memcpy(d + n, s, m);
+		n += m;
 	}
-	d[n]=0;
+	if (d) d[n] = 0;
 	return n;
 }
 static int mime_decode(char *d, char *s, int n)
@@ -249,7 +257,8 @@ static int mime_decode(char *d, char *s, int n)
 	while ((s < e)
 	&& ((s2 = strstr(s, "=?")) != NULL)) {
 		if ((m = s2 - s) > 0) {
-			n += sputsn(d+n, s, m);
+			memcpy(d + n, s, m);
+			n += m;
 		}
 		s = s2 + 2;
 		if (((s2 = strchr(s, '?')) != NULL)
@@ -257,28 +266,32 @@ static int mime_decode(char *d, char *s, int n)
 		&& ((e2 = strstr(++s2, "?=")) != NULL)
 		&& ((e2 + 2) <= e)) {
 			if ((m = e2 - s2) > 0) {
-				s = malloc(m + 1);
 				switch (tolower(q)) {
 				case 'q':
-					m = quotedprintable_decode(s, s2, m);
+					m = quotedprintable_decode(d+n, s2, m);
 					break;
 				case 'b':
-					m = base64_decode(s, s2, m);
+					m = base64_decode(d+n, s2, m);
 					break;
 				default:
 					m = 0;
 					break;
 				}
 				if (m > 0) {
-					n += to_utf8(d+n, s, m);
+					m = jconv(&s, d+n, m, 0);
+					if (m > 0) {
+						memcpy(d+n, s, m);
+						n += m;
+						free(s);
+					}
 				}
-				free(s);
 			}
 			s = e2 + 2;
 		}
 	}
 	if ((m = e - s) > 0) {
-		n += sputsn(d+n, s, m);
+		memcpy(d + n, s, m);
+		n += m;
 	}
 	d[n]=0;
 	return n;
@@ -295,12 +308,14 @@ static int entity_encode(char *d, char *s, int n)
 	n = 0;
 	while ((s < e) && ((c = *s++) != 0)) {
 		if ((es = c_to_entity(c)) != NULL) {
-			n += sputs(d + n, es);
+			if (d) strcpy(d + n, es);
+			n += strlen(es);
 		} else {
-			d[n++] = c;
+			if (d) d[n] = c;
+			n++;
 		}
 	}
-	d[n] = 0;
+	if (d) d[n] = 0;
 	return n;
 }
 
@@ -370,11 +385,11 @@ int encode(char **d, char *s, int n, int q)
 		n = quotedprintable_encode(*d, s, n);
 		break;
 	case 'm':
-		*d = malloc((n * 16) + 1);
+		*d = malloc(mime_encode(NULL, s, n) + 1);
 		n = mime_encode(*d, s, n);
 		break;
 	case 'e':
-		*d = malloc((n * 6) + 1);
+		*d = malloc(entity_encode(NULL, s, n) + 1);
 		n = entity_encode(*d, s, n);
 		break;
 	case '"':
@@ -440,7 +455,7 @@ int decode(char **d, char *s, int n, int q)
  ************************************************************/
 int entity(char **d, char *s, int n)
 {
-	*d = malloc((n * 6) + 1);
+	*d = malloc(entity_encode(NULL, s, n) + 1);
 	n = entity_encode(*d, s, n);
 	if (!n && *d) {
 		free(*d);
