@@ -663,21 +663,21 @@ static int ex_symbol(exec_t* ex)
 static int ex_stdin(exec_t* ex)
 {
 	var_t v;
-	new_file(&v, ex->in);
+	new_file(ex, &v, ex->in, NULL);
 	push_stack(ex, &v);
 	return -1;
 }
 static int ex_stdout(exec_t* ex)
 {
 	var_t v;
-	new_file(&v, ex->out);
+	new_file(ex, &v, ex->out, NULL);
 	push_stack(ex, &v);
 	return -1;
 }
 static int ex_stderr(exec_t* ex)
 {
 	var_t v;
-	new_file(&v, ex->err);
+	new_file(ex, &v, ex->err, NULL);
 	push_stack(ex, &v);
 	return -1;
 }
@@ -1315,6 +1315,13 @@ int exec_bytes(exec_t* top, char *bc, int bn)
 	ex.err = top->err;
 	ex.in  = top->in;
 	
+	if (getenv("SCRIPT_FILENAME")) {
+		ex.out = tmpfile();
+		ex.err = tmpfile();
+		ex.hh = tmpfile();
+		setupcgi(&ex, top->in);
+	}
+	
 	r = call_bytes(&ex, "main", bc, 0, NULL, &res);
 	
 	switch (ex.jmp) {
@@ -1355,6 +1362,13 @@ int exec_bytes(exec_t* top, char *bc, int bn)
 		trace_links(&ex);
 	}
 	
+	if (ex.hh) {
+		flushcgi(&ex, top->out);
+		fclose(ex.hh);
+		fclose(ex.err);
+		fclose(ex.out);
+	}
+	
 	return r;
 }
 
@@ -1406,7 +1420,7 @@ int exec_init(exec_t* ex)
 {
 	memset(ex, 0, sizeof(exec_t));
 	ex->out = stdout;
-	ex->err = stderr;
+	ex->err = stdout;
 	ex->in  = stdin;
 	return 0;
 }
