@@ -33,20 +33,23 @@ int refer_reserv(exec_t* ex, array_t *arr, var_t* obj, var_t* key)
 {
 	var_t val;
 	array_get(ex, arr, 0, obj);
-	while (arr->n > 2) {
-		array_del(ex, arr, 1, key);
-		if (!var_getItem(ex, obj, key, &val)) {
-			array_get(ex, arr, 1, &val);
-			if ((val.p == symbol_proto) || (val.p == string_proto)) {
-				new_object(ex, &val, 0);
-			} else {
-				new_array(ex, &val, 0);
-			}
-			if (!var_setItem(ex, obj, key, &val)) {
-				return 0;
+	if (arr->n > 2) { 
+		while (arr->n > 2) {
+			array_del(ex, arr, 1, key);
+			if (!var_getItem(ex, obj, key, &val)) {
+				array_get(ex, arr, 1, &val);
+				if ((val.p == symbol_proto) || (val.p == string_proto)) {
+					new_object(ex, &val, 0);
+				} else {
+					new_array(ex, &val, 0);
+				}
+				if (!var_setItem(ex, obj, key, &val)) {
+					return 0;
+				}
 			}
+			copy_var(obj, &val);
 		}
-		copy_var(obj, &val);
+		array_set(ex, arr, 0, obj);
 	}
 	array_get(ex, arr, 1, key);
 	return 1;
@@ -54,7 +57,7 @@ int refer_reserv(exec_t* ex, array_t *arr, var_t* obj, var_t* key)
 int refer_set(exec_t* ex, var_t* v, var_t* val)
 {
 	if (v->p == refer_proto) {
-		var_t r,k;
+		var_t r, k;
 		if (refer_reserv(ex, v->u.p, &r, &k)) {
 			return var_setItem(ex, &r, &k, val);
 		}
@@ -63,6 +66,11 @@ int refer_set(exec_t* ex, var_t* v, var_t* val)
 	}
 	return 0;
 }
+int is_refer_var(var_t* v)
+{
+	return ((v->p == refer_proto)
+	|| (v->p == symbol_proto));
+}
 
 /************************************************************
  *
