@@ -804,11 +804,25 @@ void regex_free(exec_t* ex, void *p)
 {
 	free_link(ex, p);
 }
-void* regex_alloc(exec_t* ex)
-{
-	return alloc_link(ex, sizeof(reg_t), regex_proto);
-}
 void regex_clear(exec_t* ex, void* p)
 {
 	reg_free(p);
 }
+
+/************************************************************
+ *
+ ************************************************************/
+int new_regex (exec_t* ex, var_t* v, char *s, int n, int i)
+{
+	reg_t* reg;
+	v->p = NULL;
+	reg = (reg_t*)alloc_link(ex, sizeof(reg_t), regex_proto);
+	if (!regex_comp(ex, reg, s, n, i)) {
+		fprintf(ex->err, "regex compile error: %.*s\n", n, s);
+		regex_free(ex, reg);
+		return 0;
+	}
+	v->p = regex_proto;
+	v->u.p = (void*)reg;
+	return 1;
+}
