@@ -16,30 +16,27 @@
 #include <unistd.h>
 #include <glob.h>
 #include <sys/stat.h>
-#include <sys/mman.h>
 #include <sys/time.h>
+#include <sys/mman.h>
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
-char* tmpbuf();
-int   sputsn(char*, char*, int);
-int   sputs(char*, char*);
-char* strndupok(char*, int);
 int   strnncmp(char*, int, char*, int);
 int   strrnncmp(char*, int, char*, int);
-int   strnncasecmp(char*, int, char*, int);
-int   strrnncasecmp(char*, int, char*, int);
+
 int   strtoh(char*, int);
-int   strtonum(char*, int, double*, int*);
-int   isoctal(int);
+
 int   iscrlf(int);
-int   istvfs(int);
+
+int   isoctal(int);
 int   xtoi(int);
 int   itoX(int);
-int   ischar(int, char*);
 int   c_to_meta(int*, int);
 int   meta_to_c(char**, int);
+
 char* c_to_entity(int);
 int   entity_to_c(char **);
-int   vmclose(char*, int, FILE*);
-char* vmopen(char*, int*, FILE**);
 
 #endif /* _UTIL_H_ */
