@@ -29,8 +29,8 @@ int string_to_symbol(exec_t* ex, char *s, int n)
 	ex->ns = realloc(ex->ns, (i + 2) * sizeof(name_t));
 	ns = ex->ns + i;
 	ns->k = k;
-	ns->s = strndupok(s, n);
-	ns->n = n;
+	ns->s = calloc(n + 1, 1);
+	memcpy(ns->s, s, ns->n = n);
 	memset(ns + 1, 0, sizeof(name_t));
 	return k;
 }
@@ -432,56 +432,60 @@ int var_callFunc(exec_t* ex, var_t* v, int sym, int argc, var_t* argv, var_t* re
 /************************************************************
  *
  ************************************************************/
-void new_null (var_t* v)
+int new_null (var_t* v)
 {
 	v->p = NULL;
+	return 1;
 }
-void new_bool(var_t* v, int i)
+int new_bool(var_t* v, int i)
 {
 	v->p = bool_proto;
 	v->u.i = i != 0;
+	return 1;
 }
-void new_char (var_t* v, int i)
+int new_char (var_t* v, int i)
 {
 	v->p = char_proto;
 	v->u.i = i & 0xFF;
+	return 1;
 }
-void new_int (var_t* v, int i)
+int new_int (var_t* v, int i)
 {
 	v->p = int_proto;
 	v->u.i = i;
+	return 1;
 }
-void new_symbol (var_t* v, int i)
+int new_symbol (var_t* v, int i)
 {
 	v->p = symbol_proto;
 	v->u.i = i;
+	return 1;
 }
-void new_float (var_t* v, double f)
+int new_float (var_t* v, double f)
 {
 	v->p = float_proto;
 	v->u.f = f;
+	return 1;
 }
-void new_label (var_t* v, char *p)
+int new_label (var_t* v, char *p)
 {
 	v->p = label_proto;
 	v->u.p = p;
+	return 1;
 }
-void new_function (var_t* v, char *p)
+int new_function (var_t* v, char *p)
 {
 	v->p = function_proto;
 	v->u.p = p;
+	return 1;
 }
-void new_class (var_t* v, char *p)
+int new_class (var_t* v, char *p)
 {
 	v->p = class_proto;
 	v->u.p = p;
+	return 1;
 }
-void new_file(var_t* v, FILE* fp)
-{
-	v->p = file_proto;
-	v->u.p = fp;
-}
-void new_string_with (exec_t* ex, var_t* v, char *s, int n)
+int new_string_with (exec_t* ex, var_t* v, char *s, int n)
 {
 	string_t* str;
 	v->p = string_proto;
@@ -490,59 +494,55 @@ void new_string_with (exec_t* ex, var_t* v, char *s, int n)
 		str->s = s;
 		str->n = n;
 	}
+	return 1;
 }
-void new_string_copy (exec_t* ex, var_t* v, char *s, int n)
+int new_string_copy (exec_t* ex, var_t* v, char *s, int n)
 {
 	string_t* str;
 	v->p = string_proto;
 	str = v->u.p = string_alloc(ex);
 	if (s && n) {
-		str->s = strndupok(s, n);
-		str->n = n;
+		str->s = calloc(n + 1, 1);
+		memcpy(str->s, s, str->n = n);
 	}
+	return 1;
 }
-void new_string (exec_t* ex, var_t* v, void *p)
+int new_string (exec_t* ex, var_t* v, void *p)
 {
 	if (!p) p = string_alloc(ex);
 	v->p = string_proto;
 	v->u.p = p;
+	return 1;
 }
-void new_regex (exec_t* ex, var_t* v, char *s, int n, int i)
-{
-	v->p = regex_proto;
-	v->u.p = regex_alloc(ex);
-	if (!regex_comp(ex, v->u.p, s, n, i)) {
-		fprintf(ex->err, "regex compile error: %.*s\n", n, s);
-		regex_free(ex, v->u.p);
-		/* make null */;
-		v->p = NULL;
-	}
-}
-void new_array (exec_t* ex, var_t* v, void *p)
+int new_array (exec_t* ex, var_t* v, void *p)
 {
 	if (!p) p = array_alloc(ex);
 	v->p = array_proto;
 	v->u.p = p;
+	return 1;
 }
-void new_refer (exec_t* ex, var_t* v, void *p)
+int new_refer (exec_t* ex, var_t* v, void *p)
 {
 	if (!p) p = refer_alloc(ex);
 	v->p = refer_proto;
 	v->u.p = p;
+	return 1;
 }
-void new_object (exec_t* ex, var_t* v, void* p)
+int new_object (exec_t* ex, var_t* v, void* p)
 {
 	if (!p) p = object_alloc(ex);
 	v->p = object_proto;
 	v->u.p = p;
+	return 1;
 }
-void new_frame (exec_t* ex, var_t* v, void *p)
+int new_frame (exec_t* ex, var_t* v, void *p)
 {
 	if (!p) {
 		p = frame_alloc(ex);
 	}
 	v->p = frame_proto;
 	v->u.p = p;
+	return 1;
 }
 
 /************************************************************
@@ -588,6 +588,10 @@ static double float_toFloat(exec_t* ex, var_t* v)
 /************************************************************
  *
  ************************************************************/
+static int symbol_toString(exec_t* ex, var_t* v, char **s, int *n)
+{
+	return symbol_to_string(ex, v->u.i, s, n);
+}
 static int bool_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
 	if (v->u.i) {
@@ -601,19 +605,24 @@ static int bool_toString(exec_t* ex, var_t* v, char **s, int *n)
 }
 static int char_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
-	return (*n = sprintf(*s = tmpbuf(), "%c", (v->u.i & 0xFF)));
+	static char p[64][4];
+	static int i = 0;
+	if (i >= 64) i=0;
+	return (*n = sprintf(*s = p[i++], "%c", (v->u.i & 0xFF)));
 }
 static int int_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
-	return (*n = sprintf(*s = tmpbuf(), "%d", v->u.i));
+	static char p[16][16];
+	static int i = 0;
+	if (i >= 16) i=0;
+	return (*n = sprintf(*s = p[i++], "%d", v->u.i));
 }
 static int float_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
-	return (*n = sprintf(*s = tmpbuf(), "%g", v->u.f));
-}
-static int symbol_toString(exec_t* ex, var_t* v, char **s, int *n)
-{
-	return symbol_to_string(ex, v->u.i, s, n);
+	static char p[8][32];
+	static int i = 0;
+	if (i >= 8) i=0;
+	return (*n = sprintf(*s = p[i++], "%g", v->u.f));
 }
 
 /************************************************************
@@ -852,9 +861,9 @@ NULL, /* eachItem */
 
 proto_t _file_proto_ = {
 "file", /* typestr */
-0, /* lock */
-NULL, /* trace */
-NULL, /* clear */
+1, /* lock */
+file_trace, /* trace */
+file_clear, /* clear */
 NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
@@ -873,9 +882,9 @@ NULL, /* eachItem */
 
 proto_t _dbm_proto_ = {
 "dbm", /* typestr */
-0, /* lock */
-NULL, /* trace */
-NULL, /* clear */
+1, /* lock */
+dbm_trace, /* trace */
+dbm_clear, /* clear */
 NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
